@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2013, Luis Pedro Coelho <luis@luispedro.org>
+# Copyright (C) 2013-2018, Luis Pedro Coelho <luis@luispedro.org>
 # vim: set ts=4 sts=4 sw=4 expandtab smartindent:
 # LICENSE: MIT
 '''
@@ -8,15 +8,33 @@ Jug.IO module
 - write_task_out: write out results, possibly with metadata.
 '''
 
-from .task import TaskGenerator
+from .task import TaskGenerator, Tasklet
+__all__ = [
+        'NoLoad',
+        'write_task_out',
+        'write_metadata',
+        'print_task_summary_table',
+        ]
 
-class NoLoad(object):
-    def __init__(self, t):
-        self.t = t
+class NoLoad(Tasklet):
+    '''
+    NoLoad can be used to decorate a Task result such that when it is passed to
+    another Task, then it is passed directly (instead of passing the result).
 
-    def __jug_hash__(self):
-        from .hash import hash_one
-        return hash_one(['NoLoad', self.t.hash()])
+    This is **for advanced usage**.
+    '''
+
+    def __init__(self, base):
+        Tasklet.__init__(self, base, 'NoLoad')
+
+    @property
+    def t(self):
+        import warnings
+        warnings.warn("Using NoLoad.t is deprecated. Use the `base` accessor", DeprecationWarning)
+        return self.base
+
+    def __getitem__(self, _):
+        raise NotImplementedError("You cannot slice a NoLoad object")
 
     def __jug_value__(self):
         return self
