@@ -3,7 +3,6 @@ import os
 
 from jug.tests.task_reset import task_reset
 from jug.tests.utils import simple_execute
-from jug import task
 import jug.jug
 
 
@@ -38,10 +37,12 @@ def test_tasklet_dependencies():
 
 
 @task_reset
-def test_tasklet_dependencies():
+def test_tasklet_slice_dependencies():
     jugfile = os.path.join(_jugdir, 'slice_task.py')
     store, space = jug.jug.init(jugfile, 'dict_store')
     simple_execute()
     assert space['z2'].value() == 0
+    assert space['z2_2'].value() == 0
+    assert space['z3'].value() == 1
 
 
