@@ -1,6 +1,6 @@
 from time import sleep
 from os import system
-from nose.tools import with_setup
+from .utils import remove_files
 
 import jug.utils
 import jug.task
@@ -10,10 +10,9 @@ from jug.backends.dict_store import dict_store
 def _remove_test_file():
     system("rm test_file")
 
-@with_setup(teardown=_remove_test_file)
+@remove_files(['test_file'])
 def test_util_timed_path():
     from jug.hash import hash_one
-    Task = jug.task.Task
     jug.task.Task.store = dict_store()
     system("touch test_file")
     t0 = jug.utils.timed_path('test_file')
